package lbi.file.format;

import java.io.*;

import lbi.utils.*;

/** From CSV (with column names in the first row) to ARFF format */
public class Csv2Arff {

    public static void cvs2arffmd(String csv, String arff) throws IOException {
        BufferedReader input = IOUtil.getReader(csv);
    	// get column names
        String[] columnNames = IOUtil.readTokens(input);
        int ncols = columnNames.length;
        // init types for each column
        CType[] ctype = new CType[ncols];
        for (int i = 0; i < ncols; ++i)
            ctype[i] = new CType();
        // scan input file
        String[] tokens;
        while ((tokens = IOUtil.readTokens(input)) != null)
            for (int i = 0; i < ncols; ++i)
                ctype[i].updateType(tokens[i]); // update type
        input.close();
        // open output
        PrintWriter output = IOUtil.getWriter(arff);
        output.println("@relation " + csv);
        for (int i = 0; i < ncols; ++i)
        	output.println("@attribute " + columnNames[i] + " " + ctype[i].getType()); // write types
        output.println("@data ");
        // re-open input
        input = IOUtil.getReader(csv);
        input.readLine(); // skip metadata
        // copy data
        String line;
        while( (line=input.readLine()) != null)
        	output.println(line);
        input.close();
        output.close();
    }

    public static void cvs2arff(String csv, String arff) throws IOException {
        BufferedReader input = IOUtil.getReader(csv);
    	// get first row
        String[] tokens = IOUtil.readTokens(input);
        int ncols = tokens.length;
        // init types for each column
        CType[] ctype = new CType[ncols];
        for (int i = 0; i < ncols; ++i)
            ctype[i] = new CType();
        // update types of first row
        for (int i = 0; i < ncols; ++i)
            ctype[i].updateType(tokens[i]); // update type
        // scan input file
        while ((tokens = IOUtil.readTokens(input)) != null)
            for (int i = 0; i < ncols; ++i)
                ctype[i].updateType(tokens[i]); // update type
        input.close();
        // open output
        PrintWriter output = IOUtil.getWriter(arff);
        output.println("@relation " + csv);
        for (int i = 0; i < ncols; ++i)
        	output.println("@attribute column-" + i + " " + ctype[i].getType()); // write types
        output.println("@data ");
        // re-open input
        input = IOUtil.getReader(csv);
        input.readLine(); // skip metadata
        // copy data
        String line;
        while( (line=input.readLine()) != null)
        	output.println(line);
        input.close();
        output.close();
    }


    public static void main(String[] args) throws IOException {
        if (args.length != 2 && args.length != 3) {
            System.out.println("Usage: java Csv2Arff [-m] csv arff");
            System.exit(0);
        }
        if( args[0].equals("-m"))
        	cvs2arffmd(args[1], args[2]);
        else
        	cvs2arff(args[0], args[1]);
     }
}
