package lbi.file.format;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.TreeSet;

import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

import lbi.utils.IOUtil;

/** From XML row to CSV format */
public class XmlRows2Csv {
	
	// retrieve all attribute names present in any row (some rows may miss an attribute)
	public static TreeSet<String> getAttributes(Element root) {
		TreeSet<String> attNames = new TreeSet<String> ();
        NodeList rows = root.getChildNodes();
        int nrows = rows.getLength();
        for(int i=0; i<nrows; ++i) {
         	Element childElement = (Element)rows.item(i);
        	NamedNodeMap atts = childElement.getAttributes();
        	int natts = atts.getLength();
        	for(int j=0; j<natts; ++j) {
        		Attr a = (Attr)atts.item(j);
        		attNames.add( a.getNodeName() );
        	}
        }
    	return attNames;
	}

	public static void main(String[] args) throws IOException, ParserConfigurationException, SAXException {
        if (args.length != 2) {
            System.out.println("Usage: java XmlRows2Csv xmlrows csv");
            System.exit(0);
        }
        // open XML
        Document d = IOUtil.getDocument(args[0]);
        Element root = d.getDocumentElement();
        // Rows list
        NodeList rows = root.getChildNodes();
        int nrows = rows.getLength();
        if( nrows == 0 )
        	return;
        TreeSet<String> attNames = getAttributes(root);
    	// Output metadata
        PrintWriter output = IOUtil.getWriter(args[1]);
        boolean first = true;
    	for(String a : attNames) {
    		if( first )
    			first = false;
    		else
    			output.print(",");
    		output.print(a);
    	}
    	output.println();
    	// output data
        for(int i=0; i<nrows; ++i) {
         	Element childElement = (Element)rows.item(i);
         	NamedNodeMap atts = childElement.getAttributes();
            first = true;
        	for(String a : attNames) {
        		if( first )
        			first = false;
        		else
        			output.print(",");
        		Node att = atts.getNamedItem(a);
        		if(att==null)
        			output.print("?");
        		else
        			output.print(att.getNodeValue());
        	}
        	output.println();
        }
	}

}
